/* =================================================================================
 *  liste.c
 * =================================================================================
 * Contiene l'implementazione di funzioni e procedure per:
 * - costruire una lista di interi
 * - stampare il contenuto di una lista di interi
 * - verificare l'ordinamento di una lista di interi
 * - calcolare l'intersezione di due liste di interi non ordinate
 * - calcolare l'intersezione di due liste di interi ordinate
 *
 *  Authors:     Irene Finocchi  (finocchi@di.uniroma1.it)
 *  Last change: 18 Aprile 2005
 *  Changed by:  Irene Finocchi
*/

#include "liste.h"

/* Pre: la lista che si costruisce e' NON vuota */
/* Post: costruisce una lista di caratteri */
ListPtr crealista() {
	ListPtr L;
	int i;

	scanf("%d",&i);
	if (i==-1) return NULL;
	L = malloc(sizeof(struct nodolista));	
	if (!L) {
		printf("Allocazione di memoria fallita nella creazione della lista");
		return NULL;
	}
	L->elem = i;
	L->next = crealista();

	return(L);
}

/* Post: stampa il contenuto della lista */
void stampaLista(ListPtr L) {
	if (L!=NULL) {
		printf("%d ",L->elem);
		stampaLista(L->next);
	}
	return;
}

/* Post: restituisce 1 se la lista e' ordinata in modo non decrescente
         e 0 altrimenti */
int verificaOrdinamento(ListPtr L) {
	if (L==NULL || L->next==NULL) return 1;
	if (L->elem <= L->next->elem) return verificaOrdinamento(L->next);
	return 0;
}

/* Post: restituisce la lista intersezione di due liste di interi non ordinate */
ListPtr intersezione(ListPtr L1, ListPtr L2) {
	ListPtr L, L3=L2;
	
	if (L1==NULL || L2==NULL) return NULL;

	while (L3!=NULL) {
		if (L1->elem==L3->elem) break;
		L3=L3->next;
	}
	
	if (L3!=NULL) {
		L = malloc(sizeof(struct nodolista));	
		if (!L) {
			printf("Allocazione di memoria fallita nella creazione della lista intersezione");
			return NULL;
		}
		L->elem = L1->elem;
		L->next = intersezione(L1->next,L2);
		return L;
	}
	else return intersezione(L1->next,L2);
}

/* Pre:  le liste L1 e L2 sono ordinate in ordine non decrescente e non 
         contengono elementi duplicati */
/* Post: restituisce la lista intersezione */
ListPtr intersezioneOrdinate(ListPtr L1, ListPtr L2) {
	ListPtr L;
	
	if (L1==NULL || L2==NULL) return NULL;
	
	printf("Chiamata con L1->elem = %d e L2->elem = %d\n",L1->elem,L2->elem);

	while (L1!=NULL && L1->elem < L2->elem) L1=L1->next;
	
	if (L1==NULL) return NULL;
	
	if (L1->elem==L2->elem) {
		L = malloc(sizeof(struct nodolista));	
		if (!L) {
			printf("Allocazione di memoria fallita nella creazione della lista intersezione");
			return NULL;
		}
		L->elem = L1->elem;
		L->next = intersezioneOrdinate(L1->next,L2->next);
		return L;
	}
	else return intersezioneOrdinate(L1,L2->next);
}









